package com.ibm.commerce.applepay.order.commands;

/*
 *-----------------------------------------------------------------
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * WebSphere Commerce
 *
 * (C) Copyright IBM Corp. 2016
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 *-----------------------------------------------------------------
 */

import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.naming.NamingException;

import com.ibm.commerce.command.CommandContext;
import com.ibm.commerce.command.CommandFactory;
import com.ibm.commerce.datatype.TypedProperty;
import com.ibm.commerce.exception.ECException;
import com.ibm.commerce.exception.ECSystemException;
import com.ibm.commerce.member.constants.ECMemberConstants;
import com.ibm.commerce.order.commands.OrderBaseCmdImpl;
import com.ibm.commerce.order.objects.OrderAccessBean;
import com.ibm.commerce.order.utils.ResolveOrdersCmd;
import com.ibm.commerce.order.utils.ResolveOrdersCmdImpl;
import com.ibm.commerce.ras.ECMessage;
import com.ibm.commerce.ras.ECMessageHelper;
import com.ibm.commerce.ras.ECTrace;
import com.ibm.commerce.ras.ECTraceIdentifiers;
import com.ibm.commerce.server.ECConstants;
import com.ibm.commerce.user.objects.AddressAccessBean;
import com.ibm.commerce.usermanagement.commands.AddressAddCmd;
import com.ibm.commerce.usermanagement.commands.AddressUpdateCmd;

/**
 * A base apple pay command.
 */
public class ApplePayOrderBaseCmdImpl extends OrderBaseCmdImpl {
	
	
	private Map<String, String> addressMap = null;
	
    private static final String DEFAULT_NICK_NAME = "ApplePhoneContact_0";
    
    private static final String NICK_NAME_PREFIX = "ApplePhoneContact_";
    
    private static final String APPLE_PAY_ADDRESS="applePayAddress";
    
    private static final char UNDERSCORE = '_';
    
    private static final String NO_URL = "NoURL";
    
    private static final String CLASS_NAME = "com.ibm.commerce.applepay.order.commands.ApplePayOrderBaseCmdImpl";
    
    
    private List<String> setNickNames = new ArrayList<String>(); //used nicknames
    
    private ArrayList<String> toTypedPropertyKey = new ArrayList<String>(){{
	    	 add(ECMemberConstants.EC_ADDR_CITY);
	    	 add(ECMemberConstants.EC_ADDR_COUNTRY);
	    	 add(ECMemberConstants.EC_ADDR_STATE);
	    	 add(ECMemberConstants.EC_ADDR_ZIPCODE);
	    	 add(ECMemberConstants.EC_ADDR_LASTNAME);
	    	 add(ECMemberConstants.EC_ADDR_FIRSTNAME);
	    	 add(ECMemberConstants.EC_ADDR_ADDRESS1);
	    	 add(ECMemberConstants.EC_ADDR_ADDRESS2);
	    	 add(ECMemberConstants.EC_ADDR_ADDRESS3);
	    	 add(ECMemberConstants.EC_ADDR_EMAIL1);
	    	 add(ECMemberConstants.EC_ADDR_MOBILEPHONE1);
    	 }
    };
    	 
   
    /**
     * Set address map.
     * @param addrMap
     */
    public void setAddressMap(Map addrMap ){
    	addressMap = addrMap;
    }
    
    /**
     * Get address map.
     * @return
     */
    public Map getAddressMap(){
    	return addressMap;
    }
	
    /**
     * Set typed property key.
     * @param list
     */
    public void setToTypedPropertyKey(ArrayList<String> list){
    	toTypedPropertyKey = list;
    }
    
    /**
     * Get typed property key.
     * @return
     */
    public List<String> getToTypedPropertyKey(){
    	return toTypedPropertyKey;
    }
    
    /**
     * Query the address id by address map.
     * @return addressId.
     * @throws ECException
     */
    protected Long queryAddressId() throws ECException{
    	final String strMethod = "queryAddressId";
    	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, getClass().getName(), strMethod);
		Map mapAddressToId = new HashMap();

		mapAddressToId = new HashMap();
		
		Enumeration en;
		try {
			en = new AddressAccessBean().findByMemberIdOrderByNickName(getCommandContext().getUserId());
			while(en.hasMoreElements()) {
				
				AddressAccessBean abAddress = (AddressAccessBean)en.nextElement();
				Long nAddressId = abAddress.getAddressIdInEJBType();
				TypedProperty typedProperty = toTypedProperty(abAddress, toTypedPropertyKey);
				
				mapAddressToId.put(typedProperty, nAddressId);
				///mapIdToAddress.put(nAddressId, typedProperty);
				setNickNames.add(abAddress.getNickName());
				
			}
		} 
		catch(CreateException e) { throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
		catch(FinderException e) { throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
		catch(NamingException e) { throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
		catch(RemoteException e) { throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
			
		TypedProperty typedProperty = new TypedProperty();
		typedProperty.putAll(addressMap);
		Long nAddressId = null;
		nAddressId = (Long)mapAddressToId.get(typedProperty);
		
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod, nAddressId);
		return nAddressId;
    }
    
	
	/**
	 * Check the address id to be updated.
	 * @return
	 * @throws ECException
	 */
	protected String checkUpdateAddressId() throws ECException{
		final String strMethod = "checkUpdateAddressId";	
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, getClass().getName(), strMethod);
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod);
		return null;
		
	}

	/**
	 * Perform the address logic. If the address is not an existed address, create a new one.
	 * If the address is an existed address but need to be updated, then update it.
	 * If the address is exactly same as the existing address, return the address id directly.
	 * 
	 * @return addressId.
	 * @throws ECException
	 */
	protected String queryAddOrUpdateAddress() throws ECException{
		
		final String strMethod = "queryAddOrUpdateAddress";
		
		TypedProperty property = (TypedProperty) getCommandContext().getRequestProperties().clone();
		
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, getClass().getName(), strMethod);
		String addressId = null;
		Long nAddressId = queryAddressId();
		if(nAddressId == null) {
			//check add a new address or update existing address
			String updateAddressId = checkUpdateAddressId();
			if(updateAddressId != null){
				TypedProperty requestProperties = new TypedProperty();;
				requestProperties.put(ECConstants.EC_FOR_USER_ID, getCommandContext().getUserId().toString());
				requestProperties.put(ECConstants.EC_ADDR_RN, updateAddressId);
				requestProperties.put(ECConstants.EC_URL, NO_URL);
				requestProperties.putAll(addressMap);
				ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, getClass().getName(), strMethod, "requestProperties=" + requestProperties);
				CommandContext commandContext = (CommandContext)getCommandContext().clone();
				commandContext.setRequestProperties(requestProperties);
				AddressUpdateCmd cmd = (AddressUpdateCmd)CommandFactory.createCommand(AddressUpdateCmd.NAME, getCommandContext().getStoreId());
				cmd.reset();
				cmd.setCommandContext(commandContext);
				cmd.setRequestProperties(requestProperties);
				cmd.execute();	//6. call AddressAddCmd to add address
				String addrId = cmd.getAddressId();
				nAddressId = new Long(addrId);
				
				try {
					//Add new address to map
					AddressAccessBean addressAB = new AddressAccessBean();
					addressAB.setInitKey_AddressId(addrId);
					addressAB.refreshCopyHelper();
					TypedProperty typedProperty = toTypedProperty(addressAB, toTypedPropertyKey);
					
				} catch(CreateException e) { throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
				catch(FinderException e) { throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
				catch(NamingException e) { throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
				catch(RemoteException e) { throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
				
				
			}else{
				String strNickName = DEFAULT_NICK_NAME;
				if (!setNickNames.isEmpty() ){
					strNickName = generateNickNameForAddress(setNickNames);
				}
				TypedProperty requestProperties = new TypedProperty();;
				requestProperties.put(ECConstants.EC_FOR_USER_ID, getCommandContext().getUserId().toString());
				requestProperties.put(ECMemberConstants.EC_ADDR_NICKNAME, strNickName);
				requestProperties.put(ECConstants.EC_URL, NO_URL);
				requestProperties.putAll(addressMap);
				ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, getClass().getName(), strMethod, "requestProperties=" + requestProperties);
				CommandContext commandContext = (CommandContext)getCommandContext().clone();
				commandContext.setRequestProperties(requestProperties);
				AddressAddCmd cmd = (AddressAddCmd)CommandFactory.createCommand(AddressAddCmd.NAME, getCommandContext().getStoreId());
				cmd.reset();
				cmd.setCommandContext(commandContext);
				cmd.setRequestProperties(requestProperties);
				cmd.execute();	//6. call AddressAddCmd to add address
				String addrId = cmd.getAddressId();
				nAddressId = new Long(addrId);
				
				
				try {
					AddressAccessBean addressAB = new AddressAccessBean();
					addressAB.setInitKey_AddressId(addrId);
					addressAB.refreshCopyHelper();
					TypedProperty typedProperty = toTypedProperty(addressAB, toTypedPropertyKey);
					//setNickNames.add(strNickName);
				} catch(CreateException e) { throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
				catch(FinderException e) { throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
				catch(NamingException e) { throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
				catch(RemoteException e) { throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
				
			}
			
			ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, getClass().getName(), strMethod, "newly created addressId=" + nAddressId);
			
		}
		if (nAddressId != null){
			addressId = nAddressId.toString();
		}
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod, addressId);
		return addressId;
	}
	
	
	
	/**
	 * Converts an {@link AddressAccessBean} to a typed property.
	 * @param aabAddress the AddressAccessBean.
	 * @return the equivalent typed property.
	 * @throws ECException
	 */
	protected TypedProperty toTypedProperty(AddressAccessBean aabAddress, List<String> propertyKeyList) throws ECException {
		final String strMethod = "toTypedProperty";
		try {
			TypedProperty typedProperty = new TypedProperty();
			for (String propertyKey : propertyKeyList){
				if (propertyKey.equals(ECMemberConstants.EC_ADDR_CITY)){
					addParameter(typedProperty, ECMemberConstants.EC_ADDR_CITY,				aabAddress.getCity()			);
					continue;
				}
				
				if (propertyKey.equals(ECMemberConstants.EC_ADDR_STATE)){
					addParameter(typedProperty, ECMemberConstants.EC_ADDR_STATE,			aabAddress.getState()			);
					continue;
				}
				if (propertyKey.equals(ECMemberConstants.EC_ADDR_COUNTRY)){
					addParameter(typedProperty, ECMemberConstants.EC_ADDR_COUNTRY,			aabAddress.getCountry()			);
					continue;
				}
				if (propertyKey.equals(ECMemberConstants.EC_ADDR_ZIPCODE)){
					addParameter(typedProperty, ECMemberConstants.EC_ADDR_ZIPCODE,			aabAddress.getZipCode()			);
					continue;
				}
				if (propertyKey.equals(ECMemberConstants.EC_ADDR_LASTNAME)){
					addParameter(typedProperty, ECMemberConstants.EC_ADDR_LASTNAME,			aabAddress.getLastName()		);
					continue;
				}
				if (propertyKey.equals(ECMemberConstants.EC_ADDR_FIRSTNAME)){
					addParameter(typedProperty, ECMemberConstants.EC_ADDR_FIRSTNAME,		aabAddress.getFirstName()		);
					continue;
				}
				
				if (propertyKey.equals(ECMemberConstants.EC_ADDR_ADDRESS1)){
					addParameter(typedProperty, ECMemberConstants.EC_ADDR_ADDRESS1,			aabAddress.getAddress1()		);
					continue;
				}
				
				if (propertyKey.equals(ECMemberConstants.EC_ADDR_ADDRESS2)){
					addParameter(typedProperty, ECMemberConstants.EC_ADDR_ADDRESS2,			aabAddress.getAddress2()		);
					continue;
				}
				if (propertyKey.equals(ECMemberConstants.EC_ADDR_ADDRESS3)){
					addParameter(typedProperty, ECMemberConstants.EC_ADDR_ADDRESS3,			aabAddress.getAddress3()		);
					continue;
				}
				
				if (propertyKey.equals(ECMemberConstants.EC_ADDR_MOBILEPHONE1)){
					addParameter(typedProperty, ECMemberConstants.EC_ADDR_MOBILEPHONE1,		aabAddress.getMobilePhone1()	);
					continue;
				}
				if (propertyKey.equals(ECMemberConstants.EC_ADDR_EMAIL1)){
					addParameter(typedProperty, ECMemberConstants.EC_ADDR_EMAIL1,		aabAddress.getEmail1()	);
					continue;
				}

			}
			
			return typedProperty;
		}
		catch(CreateException e) { throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
		catch(FinderException e) { throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
		catch(NamingException e) { throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
		catch(RemoteException e) { throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, getClass().getName(), strMethod, new Object[] { e.toString() }, e); }
	}
	/**
	 * This method generates the NickName for the newly create address record.
	 * 
	 * @param listNickNames
	 * @return
	 * @throws ECException
	 */
	protected String generateNickNameForAddress(List<String> listNickNames) throws ECException{
		final String strMethod = "generateNickNameForAddress";
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod);
		List<String> nickNames = new ArrayList<String>();
		String nickName = DEFAULT_NICK_NAME;
		Boolean reSort=false;
		for (String nk: listNickNames){
			if(nk.contains(NICK_NAME_PREFIX) ){
				nickNames.add(nk);
				if (Integer.parseInt(nk.replace(NICK_NAME_PREFIX, ""))>=10){
					reSort=true;
				}
			}
			
		}
		
		if (!nickNames.isEmpty()){
			if (!reSort){
				String lastNickName = nickNames.get(nickNames.size()-1);
				
				int tagNum = Integer.parseInt(lastNickName.replace(NICK_NAME_PREFIX, ""))+1;
				nickName = NICK_NAME_PREFIX+tagNum;
			}else{

				int size = nickNames.size();
				int number[]=new int[nickNames.size()];
				for(int i=0;i<nickNames.size();i++){
					number[i]=Integer.parseInt(nickNames.get(i).replace(NICK_NAME_PREFIX, ""));
				}
				int max=number[0]; 			  
				    for(int i=0;i<number.length;i++){  
				    	if(max<number[i]) 
				    		max=number[i]; 
				    		} 
			
				    nickName = NICK_NAME_PREFIX+(max+1);
			
			}
			
			
		}
		
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod, new Object[]{nickName});
		return nickName;
	}
	
	/**
	 * Adds a parameter to a typed property. The parameter will not be added if its value is null or empty string.
	 * @param aTypedProperty the typed property.
	 * @param astrParameter the parameter.
	 * @param astrValue the value.
	 */
	protected static void addParameter(TypedProperty aTypedProperty, String astrParameter, String astrValue) {
		if(astrValue != null) {
			astrValue = astrValue.trim();
			if(astrValue.length() != 0) {
				aTypedProperty.put(astrParameter, astrValue);
			}
		}
	}
	
	
	private OrderAccessBean [] resolveOrders(String[] orderIds) throws ECException
	{
		
		final String methodName = "resolveOrders";
		
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName);
		
		OrderAccessBean [] abOrders = null;
		if (orderIds != null){
			try
			{			
			
			
				CommandContext cmdContext = getCommandContext();
					
				ResolveOrdersCmd resolveOrdersCmd = ResolveOrdersCmdImpl.getInstance(cmdContext);
				resolveOrdersCmd.setMemberId          (cmdContext.getUserId());
				resolveOrdersCmd.setOrderAbbreviations(orderIds         );
				resolveOrdersCmd.setStoreId           (getStoreId ()         );
				resolveOrdersCmd.setStrict            (false                 ); // we should be able to work on orders owned by different stores
	
				resolveOrdersCmd.execute();
			
				abOrders = resolveOrdersCmd.getOrders();
				if (ECTrace.traceEnabled(ECTraceIdentifiers.COMPONENT_ORDER)){
					
					StringBuffer    strBuf      = new StringBuffer(100);
					for (int idx = 0; idx < abOrders.length; idx++)
					{
						strBuf.append(abOrders[idx].getOrderId() + "; ");
					}
					ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName,"Orders: " + strBuf);
				}			
			}
			catch (RemoteException e) { throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e ); }
			catch (CreateException e) { throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e ); }
			catch (NamingException e) { throw new ECSystemException(ECMessage._ERR_GENERIC         , CLASS_NAME, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e ); }
			catch (FinderException e) { throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, methodName, ECMessageHelper.generateMsgParms( e.getMessage()), e ); }
		}
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName);
		
		return abOrders;
		
	}
}
