package com.ibm.commerce.applepay.order.commands;

/*
 *-----------------------------------------------------------------
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * WebSphere Commerce
 *
 * (C) Copyright IBM Corp. 2016
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 *-----------------------------------------------------------------
 */


import java.rmi.RemoteException;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.ejb.RemoveException;
import javax.naming.NamingException;

import com.ibm.commerce.command.CommandContext;
import com.ibm.commerce.command.CommandFactory;
import com.ibm.commerce.datatype.TypedProperty;
import com.ibm.commerce.exception.ECApplicationException;
import com.ibm.commerce.exception.ECException;
import com.ibm.commerce.exception.ECSystemException;
import com.ibm.commerce.order.objects.OrderAccessBean;
import com.ibm.commerce.order.objects.OrderItemAccessBean;
import com.ibm.commerce.order.utils.OrderConstants;
import com.ibm.commerce.order.utils.ResolveOrdersCmd;
import com.ibm.commerce.order.utils.ResolveOrdersCmdImpl;
import com.ibm.commerce.orderitems.commands.OrderItemDeleteCmd;
import com.ibm.commerce.ras.ECMessage;
import com.ibm.commerce.ras.ECTrace;
import com.ibm.commerce.ras.ECTraceIdentifiers;
import com.ibm.commerce.server.ECConstants;
import com.ibm.commerce.user.objects.AddressAccessBean;
import com.ibm.commerce.usermanagement.commands.AddressDeleteCmd;

/**
 * This is the default implementation of the controller command <code>ApplePayOrderCancelCmd</code>.
 * It cancels the apple pay order..
 * 
 * 
 *<h4>Behaviour:</h4>
 * <ul>
 * 		<li>Delete orderitems</li> 
	    <li>Delete order </li> 		
	    <li>Delete Address if it was a dummy Address </li> 	
	</ul>	

 */

public class ApplePayOrderCancelCmdImpl extends ApplePayOrderBaseCmdImpl implements ApplePayOrderCancelCmd {
	 /** The IBM Copyright notice field. */
    public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;

    /** The name of this class for trace/logging/debuging purposes. */
    private static final String CLASS_NAME = "com.ibm.commerce.applepay.order.commands.ApplePayOrderCancelCmdImpl";
    
    private static final String NO_URL = "NoURL";
    
    private String orderId = null;
    
    private static final String NICK_NAME_PREFIX = "ApplePhoneContact_";
    
    
    
    
	/**
	 * This method performs server side parameter checking.
	 * @exception ECException
	 */
    public void validateParameters() throws ECException {
        String methodName = "validateParameters";
  
        try{
        	OrderAccessBean[] iabOrders = null;
        	String[] strOrderAbbreviations =
                    requestProperties.getArray(
                           OrderConstants.EC_ORDER_ID,
                           new String[] { ResolveOrdersCmd.CURRENT_PENDING_ORDERS });
             ResolveOrdersCmd cmd = ResolveOrdersCmdImpl.getInstance(getCommandContext());
             cmd.setStoreId(getCommandContext().getStoreId());
             cmd.setCommandContext(getCommandContext());
             cmd.setOrderAbbreviations(strOrderAbbreviations);
             cmd.execute();
             iabOrders = cmd.getOrders();
             if (iabOrders == null || !(iabOrders.length >0)) {
                 // throw parameter orderId is not set exception
                 throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER,CLASS_NAME,methodName,
              		   							new Object[] { "Order" });
          }else{
          	OrderAccessBean iabOrder = iabOrders[0];
          	setOrderId(iabOrder.getOrderId());
          }
          
        }catch (RemoteException e) {
            throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, methodName, e);
        } catch (CreateException e) {
            throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, methodName, e);
        } catch (FinderException e) {
            throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, methodName, e);
        } catch (NamingException e) {
            throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, CLASS_NAME, methodName, e);
        }
        
        
        ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName);
	}
    
	/**
	 * PeformExecute logic.
	 */
	public void performExecute() throws ECException{
		final String methodName = "performExecute";
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName);
		OrderAccessBean orderAB = new OrderAccessBean();
		orderAB.setInitKey_orderId(orderId);
		try {
			orderAB.refreshCopyHelper();
			OrderItemAccessBean []oiAB= orderAB.getOrderItems();
			deleteDummyAddress(orderAB);
			if(oiAB.length==1 && oiAB[0].getComment() != null){
				if(oiAB[0].getComment().equalsIgnoreCase("PDP")){
					deleteOrder(oiAB,orderId);	
				}
				
			}
			//TODO reset the orderitem address
			
			if (responseProperties == null){
				responseProperties =  new TypedProperty();
			}
			responseProperties.put("Success","Success" );
			
		} catch (RemoteException e) {
            throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, methodName, e);
        } catch (CreateException e) {
            throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, methodName, e);
        } catch (FinderException e) {
            throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, methodName, e);
        } catch (NamingException e) {
            throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, CLASS_NAME, methodName, e);
        } catch (SQLException e) {
        	throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, CLASS_NAME, methodName, e);
		} catch (RemoveException e) {
			throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, CLASS_NAME, methodName, e);		}
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName);
	}

	
	/**
	 * Delete the dummy address
	 * @param  order
	 * @throws RemoteException
	 * @throws CreateException
	 * @throws FinderException
	 * @throws NamingException
	 * @throws ECException
	 */
	public void deleteDummyAddress(OrderAccessBean order) throws RemoteException, CreateException, FinderException, NamingException, ECException {
		
		 final String methodName = "deleteDummyAddress";
		 OrderItemAccessBean[] orderItemABs = order.getOrderItems();
		 List<String> addressIds = new ArrayList<String>();
		 for (int i=0; i< orderItemABs.length; i++){
			 OrderItemAccessBean orderItemAB = orderItemABs[i];
			 String addressId = orderItemAB.getAddressId();
			 if (addressId != null && !addressIds.contains(addressId)){
				 addressIds.add(addressId);
			 }
		 }
		 
		 for (int i = 0; i< addressIds.size(); i++){
			 AddressAccessBean addressAB = new AddressAccessBean();
			 if(!addressIds.get(i).isEmpty()){
					addressAB.setInitKey_AddressId(addressIds.get(i));
					addressAB.refreshCopyHelper();
					String lastName = addressAB.getLastName();
					String status=addressAB.getStatus();
					String nickName = addressAB.getNickName();
					if (nickName.startsWith(NICK_NAME_PREFIX) && (lastName == null || lastName.trim().isEmpty()) && status != "T"){
						TypedProperty requestProperties = new TypedProperty();
						requestProperties.put(ECConstants.EC_ADDR_RN, addressAB.getAddressId());
						requestProperties.put(ECConstants.EC_URL, NO_URL);
						
						ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, getClass().getName(), methodName, "requestProperties=" + requestProperties);
						
						CommandContext commandContext = (CommandContext)getCommandContext().clone();
						commandContext.setRequestProperties(requestProperties);
						AddressDeleteCmd cmd = (AddressDeleteCmd)CommandFactory.createCommand(AddressDeleteCmd.NAME, getCommandContext().getStoreId());
						cmd.reset();
						cmd.setCommandContext(commandContext);
						cmd.setRequestProperties(requestProperties);
						cmd.execute();	//6. call AddressDelete to delete address
						setResponseProperties(cmd.getResponseProperties());
				
					}
			 }
			
		 	}
		
		}
	/**
	 * Set order id.
	 * @param orderId
	 */
	public void setOrderId(String orderId) {
		this.orderId=orderId;
		
	}

	/**
	 * Get order id.
	 * @return
	 */
	public String getOrderId() {
		return  this.orderId;
	}
	
	
	/**
	 * Delete the order which created from PDP page
	 * @param odAB
	 * @param orderId
	 * @throws RemoteException
	 * @throws CreateException
	 * @throws FinderException
	 * @throws NamingException
	 * @throws ECSystemException
	 * @throws ECException
	 * @throws SQLException
	 * @throws RemoveException
	 */
	public void deleteOrder(OrderItemAccessBean[] odAB,String orderId) throws RemoteException, CreateException, FinderException, NamingException, ECSystemException, ECException, SQLException, RemoveException {
    	 String[] removedOrderItemIds = new String[odAB.length] ;
		for (int i = 0; i<odAB.length; i++){
			removedOrderItemIds[i] = ((OrderItemAccessBean)odAB[i]).getOrderItemId().toString();
    	 }
		 OrderItemDeleteCmd cmd =
                 (OrderItemDeleteCmd) CommandFactory.createCommand(
                        OrderItemDeleteCmd.NAME,
                        getCommandContext()
                               .getStore()
                               .getStoreEntityIdInEJBType());
              cmd.setCommandContext(getCommandContext());
              cmd.setOrderItemId(removedOrderItemIds);
              cmd.execute();
              OrderAccessBean deleteOrderAccessBean= new OrderAccessBean();
              deleteOrderAccessBean.setInitKey_orderId(orderId);
              deleteOrderAccessBean.refreshCopyHelper();
              deleteOrderAccessBean.getEJBRef().remove();
              
	}
	
}
