package com.ibm.commerce.applepay.order.commands;

/*
 *-----------------------------------------------------------------
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * WebSphere Commerce
 *
 * (C) Copyright IBM Corp. 2016
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 *-----------------------------------------------------------------
 */

import java.math.BigDecimal;
import java.rmi.RemoteException;
import java.util.HashMap;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.naming.NamingException;

import com.ibm.commerce.command.CommandContext;
import com.ibm.commerce.command.CommandFactory;
import com.ibm.commerce.command.ECCommand;
import com.ibm.commerce.datatype.TypedProperty;
import com.ibm.commerce.edp.commands.PIAddCmd;
import com.ibm.commerce.exception.ECApplicationException;
import com.ibm.commerce.exception.ECException;
import com.ibm.commerce.exception.ECSystemException;
import com.ibm.commerce.order.commands.OrderBaseCmdImpl;
import com.ibm.commerce.order.commands.OrderPrepareCmd;
import com.ibm.commerce.order.objects.OrderAccessBean;
import com.ibm.commerce.order.utils.OrderConstants;
import com.ibm.commerce.order.utils.ResolveOrdersCmd;
import com.ibm.commerce.order.utils.ResolveOrdersCmdImpl;
import com.ibm.commerce.ras.ECMessage;
import com.ibm.commerce.ras.ECMessageHelper;
import com.ibm.commerce.ras.ECTrace;
import com.ibm.commerce.ras.ECTraceIdentifiers;
import com.ibm.commerce.server.ECConstants;


/**
 * This is the default implementation of the controller command <code>ApplePayOrderPrepareCmd</code>.
 * It prepares the payment sheet data for Apple Pay shopping flow..
 * 
 * 
 *<h4>Behaviour:</h4>
 * <ul>
 * 		<li>If addUnboundPI is true, then add the unbound payment instruction for payment typed promotion.</li> 
	    <li>Call OrderPrepareCmd to prepare the apple pay order. </li> 		
	</ul>	

 */
public class ApplePayOrderPrepareCmdImpl  extends OrderBaseCmdImpl implements
		ApplePayOrderPrepareCmd {

	/**	
	 * The IBM copyright notice field.
	 */
	public static final String	COPYRIGHT= com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
	private static final String CLASS_NAME = "com.ibm.commerce.applepay.order.commands.ApplePayOrderPrepareCmdImpl";
	
	private static final String applePayPaymentMethodId = "ApplePay";
	
	private final static String ADD_UNBOUND_PI = "addUnboundPI";
	
	private static final String UNBOUND_PI_ID = "unboundPIId";
	
	private OrderAccessBean iabOrder = null;
	private String orderId = null;
	private String payMethodId = null;
	private String billAddressId = null;
	
	private HashMap hashCustomProperties = new HashMap();
	
	private boolean addUnboundPI = false;
	
	
	/**
	 * Set order id.
	 */
	public void setOrderId(String iorderId) {
		orderId = iorderId;
	}
	

	/**
	 * Set payment method id.
	 */
	public void setPayMethodId(String ipayMethodId) {
		payMethodId = ipayMethodId;
	}


	/**
	 * Set billing address id.
	 */
	public void setBillAddressId(String ibillAddressId) {
		billAddressId = ibillAddressId;
	}
	
	/**
	 * Get order id.
	 */
	public String getOrderId() {
		
		return orderId;
	}


	/**
	 * Get payment method id.
	 */
	public String getPayMethodId() {
		return payMethodId;
	}


	/**
	 * Get billing address id.
	 */
	public String getBillAddressId() {
		return billAddressId;
	}


	/**
	 * Set customization hash map.
	 */
	public void setCustomHashMap(HashMap iHshCustomProperties) {
		hashCustomProperties = iHshCustomProperties;
	}


	/**
	 * Get customization hash map.
	 */
	public HashMap getCustomHashMap() {
		return hashCustomProperties;
	}

	/**
	 * Set add unbound PI.
	 */
	public void setAddUnboundPI(boolean ifAddUnboundPI){
		addUnboundPI = ifAddUnboundPI;
	}
	
	/**
	 * Get add unbound PI.
	 */
	public boolean getAddUnboundPI(){
		return addUnboundPI;
	}
	
	/**
	 * The WebController calls the setRequestProperties method before invoking the execute method in this command.
	 * @param aReqParms request properties.
	 * @exception ECException
	 */
	public void setRequestProperties(TypedProperty aRequestProperties)
	           throws ECException {
	    	
	    	final String strMethod = "setRequestProperties";
	    	ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod);
	        super.setRequestProperties(aRequestProperties);
	        
	        addUnboundPI = this.commandContext
					.getRequestProperties().getBoolean(
							ADD_UNBOUND_PI, false);
	        
	        ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod, orderId);
	}
	
	
	/**
     * Resolve the orders to be processed from the request.
     * <br>
     * @exception ECException
     * @see ECCommand#validateParameters()
     */
	public void validateParameters() throws ECException {
        String methodName = "validateParameters";
        ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName);
        try{
        	OrderAccessBean[] iabOrders = null;
        	if (requestProperties != null) {
            	String[] strOrderAbbreviations =
                        requestProperties.getArray(
                               OrderConstants.EC_ORDER_ID,
                               new String[] { ResolveOrdersCmd.CURRENT_PENDING_ORDERS });
                 ResolveOrdersCmd cmd = ResolveOrdersCmdImpl.getInstance(getCommandContext());
                 cmd.setStoreId(getCommandContext().getStoreId());
                 cmd.setCommandContext(getCommandContext());
                 cmd.setOrderAbbreviations(strOrderAbbreviations);
                 cmd.execute();
                 iabOrders = cmd.getOrders();
                 
 	        	
 	        	payMethodId = requestProperties.getString(ECConstants.EC_EDP_PAYMTHDID, applePayPaymentMethodId);
 	        	if (iabOrders == null || !(iabOrders.length >0)) {
                    // throw parameter orderId is not set exception
                    throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER,CLASS_NAME,methodName,
                 		   							new Object[] { "Order" });
             }else{
             	iabOrder = iabOrders[0];
             	setOrderId(iabOrder.getOrderId());
             }
            }
            
        }catch (RemoteException e) {
            throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, methodName, e);
        } catch (CreateException e) {
            throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, methodName, e);
        } catch (FinderException e) {
            throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, methodName, e);
        } catch (NamingException e) {
            throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, CLASS_NAME, methodName, e);
        }
        ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName);

	}
	
	/**
	 * The performExecute logic.
	 */
	public void performExecute() throws ECException {
		final String strMethod = "performExecute";
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod);
		try{
			
			Long iabOrderId = iabOrder.getOrderIdInEJBType(); 
			Long piId = null;
			if (addUnboundPI){
				//Add PI first, this is for the payment based promotion usage
				piId = addUnboundPIForApplePay();
			
			}
		
			//Call OrderPrepare to recalculate the order, reserve inventory..etc.
			prepareOrderForApplePay();
			
			if (responseProperties == null){
				responseProperties = new TypedProperty();
			}
			responseProperties.put(ECConstants.EC_ORDER_RN, orderId);
			if (piId != null){
				responseProperties.put(UNBOUND_PI_ID, piId);
			}
			
			
		}catch (RemoteException e)
		{
			throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, strMethod, ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		catch (CreateException e)
		{
			throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, strMethod, ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		catch (NamingException e)
		{
			throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, CLASS_NAME, strMethod, ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		catch (FinderException e)
		{
			throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, strMethod, ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod);
	}
	
	/**
	 * Add unbound PI for apple pay if promotion typed promotion enabled.
	 * 
	 * @return
	 * @throws ECException
	 */
	protected Long addUnboundPIForApplePay() throws ECException{
		final String strMethod = "addUnboundPIForApplePay";
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod);
		Long piId = null;
		try{
			TypedProperty tempRequestProperties = (TypedProperty)getCommandContext().getRequestProperties().clone();
			tempRequestProperties.put(ECConstants.EC_EDP_PAYMTHDID,payMethodId);
			tempRequestProperties.put(ECConstants.EC_EDP_PIAMOUNT,new BigDecimal(0));
			tempRequestProperties.put(ECConstants.EC_EDP_ORDERID,iabOrder.getOrderId());
			tempRequestProperties.put("unbound",true);
			
			tempRequestProperties.put(ECConstants.EC_URL, "PIAdd"); // just pass validation
			PIAddCmd aPIAddCmd = (PIAddCmd) CommandFactory.createCommand(PIAddCmd.NAME, getStoreId());
			aPIAddCmd.setAccCheck(false);
			aPIAddCmd.setCommandContext((CommandContext)getCommandContext().clone());
			aPIAddCmd.setRequestProperties(tempRequestProperties);
			aPIAddCmd.execute();
			piId = aPIAddCmd.getAddedPaymentInstructionId();
		}catch (RemoteException e)
		{
			throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, strMethod, ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		catch (CreateException e)
		{
			throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, strMethod, ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		catch (NamingException e)
		{
			throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, CLASS_NAME, strMethod, ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		catch (FinderException e)
		{
			throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, strMethod, ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod, piId);
		return piId;
	}
	
	
	/**
	 * Call order calculation command.
	 * 
	 * @param property
	 *            The property. Cannot be null.
	 * 
	 * @throws ECException
	 */
	/*protected void calculateOrder() throws ECException {
		final String strMethod = "calculateOrder";
		
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod);
		
		try {
			Integer[] nUsageIds = CalculationRegistry.getInstance().getUsageIds(iabOrder.getStoreEntityIdInEJBType());
			CalculateOrderCmd calculateOrder = (CalculateOrderCmd) CommandFactory.createCommand(CalculateOrderCmd.NAME,
		            getStoreId());
			calculateOrder.setOrders(new OrderAccessBean[] {iabOrder});
			calculateOrder.setUsageIds(nUsageIds);
			calculateOrder.setDoFreeGift(false);
			calculateOrder.setCommandContext(getCommandContext());
			calculateOrder.execute();
		} catch (RemoteException e)
		{
			throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, strMethod, ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		catch (CreateException e)
		{
			throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, strMethod, ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		catch (NamingException e)
		{
			throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, CLASS_NAME, strMethod, ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		catch (FinderException e)
		{
			throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, strMethod, ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		

		ECTrace
				.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME,
						strMethod);
	}*/
	
	/**
	 * Prepare the apple pay cart.
	 * 
	 * @throws ECException
	 */
	protected void prepareOrderForApplePay() throws ECException{
		
		final String strMethod = "prepareOrderForApplePay";
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod);
		try{ 
			OrderPrepareCmd orderPrepare = (OrderPrepareCmd) CommandFactory.createCommand(OrderPrepareCmd.NAME, getCommandContext().getStore().getStoreEntityIdInEJBType());
			CommandContext cmdCtx = (CommandContext) getCommandContext().clone();
			orderPrepare.setCommandContext(cmdCtx); 		       
			TypedProperty requestProperties = cmdCtx.getRequestProperties(); // shallow clone should not be too expensive
			requestProperties.put(OrderConstants.EC_ORDER_ID, orderId);
			requestProperties.put("URL", "/"); // URL is a mandatory parameter for OrderPrepare command
			requestProperties.put("reverse", "*n");
			orderPrepare.setRequestProperties(requestProperties);
			orderPrepare.execute();	
			responseProperties = orderPrepare.getResponseProperties();
		}catch (RemoteException e)
		{
			throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, strMethod, ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		catch (CreateException e)
		{
			throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, strMethod, ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		catch (NamingException e)
		{
			throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, CLASS_NAME, strMethod, ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		catch (FinderException e)
		{
			throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, strMethod, ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME,strMethod);
		
	}
	
	/*protected void removePIForApplePay(Long piId) throws ECException{
		final String strMethod = "removePIForApplePay";
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod, new Object[]{piId});
		PIRemoveCmd aPIRemoveCmd = (PIRemoveCmd) CommandFactory.createCommand(PIRemoveCmd.NAME, getStoreId());
		aPIRemoveCmd.setCommandContext(getCommandContext());
		
		TypedProperty requestProperties = (TypedProperty)getCommandContext().getRequestProperties().clone(); // shallow clone should not be too expensive
		
		requestProperties.put(OrderConstants.EC_ORDER_ID, orderId);
		requestProperties.put(ECConstants.EC_EDP_PIID, piId);
		requestProperties.put(ECConstants.EC_URL, "PIRemove"); // just pass validation
		aPIRemoveCmd.setRequestProperties(requestProperties);
		aPIRemoveCmd.execute();
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME,strMethod);
	}
	*/
	/*protected Long addPIForApplePay() throws ECException{
		final String strMethod = "addPIForApplePay"; 
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod);
		Long newAddedPIId = null;
		try{
			
			BigDecimal totalAmount = CalculationHelper.getInstance().getOrderTotalAmount(iabOrder, getCommandContext());
			ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod, "orderTotalAmount = "+totalAmount);
			
			TypedProperty tempRequestProperties = (TypedProperty)getCommandContext().getRequestProperties().clone();
			if (billAddressId == null){
				tempRequestProperties.put(BILLING_ADDRESS_ID_PROPERTY_NAME, iabOrder.getAddressId());
			}
			tempRequestProperties.put(BILLING_ADDRESS_ID_PROPERTY_NAME, iabOrder.getAddressId());
			tempRequestProperties.put(ECConstants.EC_EDP_PAYMTHDID,payMethodId);
			tempRequestProperties.put(ECConstants.EC_EDP_PIAMOUNT,totalAmount);
			tempRequestProperties.put(ECConstants.EC_EDP_ORDERID, orderId);
			tempRequestProperties.put(ECConstants.EC_URL, "PIAdd"); // just pass validation
					
			PIAddCmd aPIAddCmd = (PIAddCmd) CommandFactory.createCommand(PIAddCmd.NAME, getStoreId());
			aPIAddCmd.setAccCheck(false);
			aPIAddCmd.setCommandContext((CommandContext)getCommandContext().clone());
			aPIAddCmd.setRequestProperties(tempRequestProperties);
			aPIAddCmd.execute();
			// The new added payment instruction ID will be put into the request properties with name "paymentInstructionId_0".
			// This logic is for addressing the PCI issue.
			newAddedPIId = aPIAddCmd.getAddedPaymentInstructionId();
		}catch (RemoteException e)
		{
			throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, strMethod, ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		catch (CreateException e)
		{
			throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, strMethod, ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		catch (NamingException e)
		{
			throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, CLASS_NAME, strMethod, ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		catch (FinderException e)
		{
			throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, strMethod, ECMessageHelper.generateMsgParms(e.getMessage()), e);
		}
		
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME,strMethod, newAddedPIId);
		return newAddedPIId;
	}*/

	
	/*protected  void updateInventory()throws ECException
	{
		final String methodName = "updateInventory";
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName);
		DoInventoryActionCmd allocateInvCmd = (DoInventoryActionCmd) CommandFactory.createCommand(DoInventoryActionCmd.NAME, getStoreId());
		allocateInvCmd.setAction(DoInventoryActionCmd.ALLOCATE_INVENTORY);			
		allocateInvCmd.setCallerCommandName(DoInventoryActionCmd.ORDER_PROCESS);
		allocateInvCmd.setOrderId(new String[]{orderId});
		CommandContext cmdCtx = (CommandContext) getCommandContext().clone();		       
		TypedProperty requestProperties = cmdCtx.getRequestProperties(); // shallow clone should not be too expensive
		requestProperties.put(OrderConstants.EC_ORDER_ID, orderId);
		requestProperties.put(OrderConstants.INVENTORY_VALIDATION, true);
		allocateInvCmd.setCommandContext(cmdCtx);
		allocateInvCmd.execute();
				
	    ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName);
	}*/

	
}
