package com.ibm.commerce.applepay.order.commands;

/*
 *-----------------------------------------------------------------
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * WebSphere Commerce
 *
 * (C) Copyright IBM Corp. 2016
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 *-----------------------------------------------------------------
 */


import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Map;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.naming.NamingException;

import com.ibm.commerce.command.CommandContext;
import com.ibm.commerce.command.CommandFactory;
import com.ibm.commerce.datatype.TypedProperty;
import com.ibm.commerce.edp.commands.PIRemoveCmd;
import com.ibm.commerce.exception.ECApplicationException;
import com.ibm.commerce.exception.ECException;
import com.ibm.commerce.exception.ECSystemException;
import com.ibm.commerce.member.constants.ECMemberConstants;
import com.ibm.commerce.order.commands.CheckInStorePickupCmd;
import com.ibm.commerce.order.commands.OrderProcessCmd;
import com.ibm.commerce.order.objects.OrderAccessBean;
import com.ibm.commerce.order.objects.OrderItemAccessBean;
import com.ibm.commerce.order.utils.OrderConstants;
import com.ibm.commerce.order.utils.ResolveOrdersCmd;
import com.ibm.commerce.order.utils.ResolveOrdersCmdImpl;
import com.ibm.commerce.ras.ECMessage;
import com.ibm.commerce.ras.ECTrace;
import com.ibm.commerce.ras.ECTraceIdentifiers;
import com.ibm.commerce.server.ECConstants;
import com.ibm.commerce.user.objects.AddressAccessBean;

/**
 * This is the default implementation of the controller command <code>ApplePayOrderProcessCmd</code>.
 * It processes the apple pay order..
 * 
 * 
 *<h4>Behaviour:</h4>
 * <ul>
 * 		<li>If unbound PI id is not null, then remove the unbound PI.</li> 
	    <li>Call OrderProcesCmd to process the apple pay order. </li> 		
	</ul>	

 */

public class ApplePayOrderProcessCmdImpl extends
ApplePayOrderBaseCmdImpl implements ApplePayOrderProcessCmd {

	 /** The IBM Copyright notice field. */
    public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;

    /** The name of this class for trace/logging/debuging purposes. */
    private static final String CLASS_NAME = "com.ibm.commerce.applepay.order.commands.ApplePayOrderProcessCmdImpl";
    
    private static final String applePayPaymentMethodId = "ApplePay";
    
    private static final String UNBOUND_PI_ID = "unboundPIId";
    
    private final static String BILLING_ADDRESS_ID_PROPERTY_NAME = "billing_address_id";
    
    private Map<String, String> addressMapForShipping = null;
    
    private Map<String, String> addressMapForBilling = null;
    
    private Map<String, String> paymentToken = null;
    
	private String billAddressId = null;
	
    private String unboundPIId = null;
    
    private String orderId = null;
    
    private static final String APPLE_PAY_SHIPPING_ADDRESS="shippingContact";
    
    private static final String APPLE_PAY_BILLING_ADDRESS="billingContact";
    
    private static final String APPLE_PAY_PAYMENT_TOKEN = "paymentToken";
    
    private ArrayList<String> toTypedPropertyKeyForBilling = new ArrayList<String>(){{
	 add(ECMemberConstants.EC_ADDR_CITY);
	 add(ECMemberConstants.EC_ADDR_COUNTRY);
	 add(ECMemberConstants.EC_ADDR_STATE);
	 add(ECMemberConstants.EC_ADDR_ZIPCODE);
	 add(ECMemberConstants.EC_ADDR_LASTNAME);
	 add(ECMemberConstants.EC_ADDR_FIRSTNAME);
	 add(ECMemberConstants.EC_ADDR_ADDRESS1);
	 add(ECMemberConstants.EC_ADDR_ADDRESS2);
	 add(ECMemberConstants.EC_ADDR_ADDRESS3);
}
};
   /**
    * Set order id.
    */
	public void setOrderId(String iorderId) {
		orderId = iorderId;
	}


	/**
	 * Get order id.
	 */
	public String getOrderId() {
		return orderId;
	}


	/**
	 * Set unbound PI id.
	 */
	public void setUnboundPIId(String piId) {
		unboundPIId = piId;
	}


	/**
	 * Get unbound PI id.
	 */
	public String getUnboundPIId() {
		return unboundPIId;
	}
	
	
	/**
	 * This method performs server side parameter checking.
	 * @exception ECException
	 */
    public void validateParameters() throws ECException {

        String methodName = "validateParameters";
        try{
        	OrderAccessBean[] iabOrders = null;
        	if (requestProperties != null) {
            	String[] strOrderAbbreviations =
                        requestProperties.getArray(
                               OrderConstants.EC_ORDER_ID,
                               new String[] { ResolveOrdersCmd.CURRENT_PENDING_ORDERS });
                 ResolveOrdersCmd cmd = ResolveOrdersCmdImpl.getInstance(getCommandContext());
                 cmd.setStoreId(getCommandContext().getStoreId());
                 cmd.setCommandContext(getCommandContext());
                 cmd.setOrderAbbreviations(strOrderAbbreviations);
                 cmd.execute();
                 iabOrders = cmd.getOrders();
                 
            }
            if (iabOrders == null || !(iabOrders.length >0)) {
                   // throw parameter orderId is not set exception
                   throw new ECApplicationException(ECMessage._ERR_BAD_MISSING_CMD_PARAMETER,CLASS_NAME,methodName,
                		   							new Object[] { "Order" });
            }else{
            	OrderAccessBean iabOrder = iabOrders[0];
            	setOrderId(iabOrder.getOrderId());
            }
            
            setUnboundPIId( requestProperties.getString(UNBOUND_PI_ID, null));
            
        }catch (RemoteException e) {
            throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, methodName, e);
        } catch (CreateException e) {
            throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, methodName, e);
        } catch (FinderException e) {
            throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, methodName, e);
        } catch (NamingException e) {
            throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, CLASS_NAME, methodName, e);
        }
        ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName);
	}
    
	/**
	 * PeformExecute logic.
	 */
	public void performExecute() throws ECException{
		final String methodName = "performExecute";
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName);
		if (unboundPIId != null){
			removePIForApplePay();
		}
		
		try {
			boolean isBOPIS = false;
			//check BOPIS
			OrderAccessBean orderAB = new OrderAccessBean();
			orderAB.setInitKey_orderId(getOrderId());
			orderAB.refreshCopyHelper();
			OrderItemAccessBean[] oiABs = orderAB.getOrderItems();
			if (oiABs != null && oiABs.length > 0){
				OrderItemAccessBean oiAB = oiABs[0];
				isBOPIS = isInStorePickupOrderItem(oiAB);
			}
			//check shipping address
			if (!isBOPIS){
				setAddressMap((Map<String, String>)addressMapForShipping);
				String ShippingId=queryAddOrUpdateAddress();
			    updateOrderAddressId(ShippingId);
			}
			
			//check billing address
			setToTypedPropertyKey(toTypedPropertyKeyForBilling);
			setAddressMap(addressMapForBilling);
			billAddressId=queryAddOrUpdateAddress();
			
			callOrderProcess();
			if (responseProperties == null){
				responseProperties =  new TypedProperty();
			}
			responseProperties.put(ECConstants.EC_ORDER_RN, getOrderId());
		}catch (CreateException e) {
            throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, getClass().getName(),
                    methodName, new Object[] {e.toString()}, e);
        } catch (FinderException e) {
                throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, getClass().getName(),
                    methodName, new Object[] {e.toString()}, e);
        } catch (NamingException e) {
                throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, getClass().getName(),
                    methodName, new Object[] {e.toString()}, e);
        } catch (RemoteException e) {
                throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, getClass().getName(),
                    methodName, new Object[] {e.toString()}, e);
        }
		
		
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName);
	}

	/**
	 * Remove the unbound PI for apple pay.
	 * @throws ECException
	 */
	protected void removePIForApplePay() throws ECException{
		final String strMethod = "removePIForApplePay";
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, strMethod);
		PIRemoveCmd aPIRemoveCmd = (PIRemoveCmd) CommandFactory.createCommand(PIRemoveCmd.NAME, getStoreId());
		aPIRemoveCmd.setCommandContext(getCommandContext());
		
		TypedProperty requestProperties = (TypedProperty)getCommandContext().getRequestProperties().clone(); // shallow clone should not be too expensive
		
		requestProperties.put(OrderConstants.EC_ORDER_ID, orderId);
		requestProperties.put(ECConstants.EC_EDP_PIID, unboundPIId);
		requestProperties.put(ECConstants.EC_URL, "PIRemove"); // just pass validation
		aPIRemoveCmd.setRequestProperties(requestProperties);
		aPIRemoveCmd.execute();
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME,strMethod);
	}
	
	/**
	 * Call OrderProcess command to process the apple pay order.
	 * 
	 * @throws ECException
	 */
	protected void callOrderProcess() throws ECException{
		
		final String methodName = "callOrderProcess";
		
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName);
		
		TypedProperty property = (TypedProperty) getCommandContext().getRequestProperties().clone();
		CommandContext tempCmdContext = getCommandContext();
		property.put(ECConstants.EC_URL,"/");
		property.put(ECConstants.EC_STORE_ID, this.getCommandContext().getStoreId());
		String payMethodId = property.getString(ECConstants.EC_EDP_PAYMTHDID, applePayPaymentMethodId);
		property.put(ECConstants.EC_EDP_PAYMTHDID,payMethodId);
		
		property.put(ECConstants.EC_ORDER_RN, orderId);
        
		property.put("billing_address_id", billAddressId);
		//add paymentToken to requestProperties
		property.putAll(paymentToken);
		
		OrderProcessCmd orderProcess =(OrderProcessCmd) CommandFactory.createCommand(OrderProcessCmd.NAME,	getCommandContext().getStoreId());
		if (orderProcess == null) {
			ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, this.getClass().getName(), methodName, "No OrderProcessCmd instance");
			throw new ECApplicationException(ECMessage._ERR_CMD_CMD_NOT_FOUND, this.getClass().getName(), methodName);
		}
		tempCmdContext.setRequestProperties(property);
		
		orderProcess.setCommandContext(tempCmdContext);
		orderProcess.setRequestProperties(property);
		orderProcess.execute();
		//set response
		setResponseProperties(orderProcess.getResponseProperties());
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName);
		
	}
	/**
	 * The WebController calls the setRequestProperties method before invoking the execute method in this command.
	 * @param aReqParms request properties.
	 * @exception ECException
	 */
	public void setRequestProperties(TypedProperty aRequestProperties) throws ECException {
		
		  final String methodName = "setRequestProperties";
	        ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName);
	        if (ECTrace.traceEnabled(ECTraceIdentifiers.COMPONENT_ORDER)) {
	            ECTrace.trace(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName,
	                "The request properties:"
	                    + (aRequestProperties == null ? "null" : aRequestProperties.toString()));
	        }
	        requestProperties = aRequestProperties;
	        if (requestProperties.containsKey(APPLE_PAY_SHIPPING_ADDRESS)){
	        	addressMapForShipping = (Map<String, String>) requestProperties.get(APPLE_PAY_SHIPPING_ADDRESS);
				requestProperties.remove(APPLE_PAY_SHIPPING_ADDRESS);
			}
	        
	      
	        if (requestProperties.containsKey(APPLE_PAY_BILLING_ADDRESS)){
	        	addressMapForBilling = (Map<String, String>) requestProperties.get(APPLE_PAY_BILLING_ADDRESS);
				requestProperties.remove(APPLE_PAY_BILLING_ADDRESS);
			}
			
	        if (requestProperties.containsKey(APPLE_PAY_PAYMENT_TOKEN)){
	        	paymentToken = (Map<String, String>) requestProperties.get(APPLE_PAY_PAYMENT_TOKEN);
				requestProperties.remove(APPLE_PAY_PAYMENT_TOKEN);
			}
			
        ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName);
	}
	
	
	public void updateOrderAddressId(String shippingId) throws ECException {
		final String methodName = "updateOrderAddress";
		
		OrderAccessBean orderAB = new OrderAccessBean();
		orderAB.setInitKey_orderId(orderId);
		try {
			orderAB.refreshCopyHelper();
			OrderItemAccessBean oiAB= orderAB.getOrderItems()[0];
			if(!(shippingId.equals(oiAB.getAddressId()))){
				oiAB.setAddressId(shippingId);
				oiAB.commitCopyHelper();
			}
			
		} catch (RemoteException e) {
            throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, methodName, e);
        } catch (CreateException e) {
            throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, methodName, e);
        } catch (FinderException e) {
            throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, methodName, e);
        } catch (NamingException e) {
            throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, CLASS_NAME, methodName, e);
        }
	}
	
	protected String checkUpdateAddressId() throws ECException{
		final String methodName = "checkUpdateAddressId";
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName);
		String updateAddrId = null;
		OrderAccessBean orderAB = new OrderAccessBean();
		orderAB.setInitKey_orderId(orderId);
		try {
			orderAB.refreshCopyHelper();
			OrderItemAccessBean oiAB= orderAB.getOrderItems()[0];
			
			
			AddressAccessBean addressAB = new AddressAccessBean();
			addressAB.setInitKey_AddressId(oiAB.getAddressId());
			addressAB.refreshCopyHelper();
			String lastName = addressAB.getLastName();
			if (lastName == null ){
				updateAddrId = oiAB.getAddressId();
			}
			
		} catch (RemoteException e) {
            throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, methodName, e);
        } catch (CreateException e) {
            throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, methodName, e);
        } catch (FinderException e) {
            throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, methodName, e);
        } catch (NamingException e) {
            throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, CLASS_NAME, methodName, e);
        }
        
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_ORDER, CLASS_NAME, methodName, updateAddrId);
		return updateAddrId;
	}
	
	/**
     * This method calls {@link CheckInStorePickupCmd} to check whether the
     * given OrderItem is a BOPIS item or not.
     * 
     * @param abOrderItem
     * @return True if the given orderItem is a BOPIS item.
     * @throws ECException
     */
    private boolean isInStorePickupOrderItem(OrderItemAccessBean abOrderItem) throws ECException {
        CheckInStorePickupCmd checkInStorePickupCmd =
            (CheckInStorePickupCmd) CommandFactory.createCommand(CheckInStorePickupCmd.NAME,
                getCommandContext().getStoreId());
        checkInStorePickupCmd.setCommandContext(getCommandContext());
        checkInStorePickupCmd.setOrderItem(abOrderItem);
        checkInStorePickupCmd.execute();
        return checkInStorePickupCmd.isInStorePickupOrderItem();
    }

	
}
