package com.ibm.commerce.applepay.order.facade.server.commands;

/*
 *-----------------------------------------------------------------
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * WebSphere Commerce
 *
 * (C) Copyright IBM Corp. 2016
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 *-----------------------------------------------------------------
 */

import java.math.BigDecimal;
import java.math.BigInteger;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.naming.NamingException;

import com.ibm.commerce.command.CommandContext;
import com.ibm.commerce.component.contextservice.ActivityToken;
import com.ibm.commerce.component.contextservice.BusinessContextServiceFactory;
import com.ibm.commerce.component.contextserviceimpl.BusinessContextInternalService;
import com.ibm.commerce.context.baseimpl.ContextHelper;
import com.ibm.commerce.contract.objects.BusinessPolicyAccessBean;
import com.ibm.commerce.exception.ECApplicationException;
import com.ibm.commerce.exception.ECException;
import com.ibm.commerce.exception.ECSystemException;
import com.ibm.commerce.foundation.common.datatypes.ContactInfoIdentifierType;
import com.ibm.commerce.foundation.common.datatypes.DescriptionType;
import com.ibm.commerce.foundation.common.datatypes.OrderItemIdentifierType;
import com.ibm.commerce.foundation.common.datatypes.UserDataType;
import com.ibm.commerce.foundation.common.util.logging.LoggingHelper;
import com.ibm.commerce.fulfillment.beans.OrderItemShippingModeListDataBean;
import com.ibm.commerce.fulfillment.beans.ShippingModeDataBean;
import com.ibm.commerce.fulfillment.objects.ShippingModeAccessBean;
import com.ibm.commerce.fulfillment.objects.ShippingModeDescriptionAccessBean;
import com.ibm.commerce.fulfillment.objsrc.ShippingModeCache;
import com.ibm.commerce.fulfillment.objsrc.ShippingModeDescriptionCache;
import com.ibm.commerce.oagis9.datatypes.GetType;
import com.ibm.commerce.order.beans.OrderDataBean;
import com.ibm.commerce.order.beans.OrderItemDataBean;
import com.ibm.commerce.order.facade.OrderFacadeConstants;
import com.ibm.commerce.order.facade.datatypes.OrderItemShippingInfoType;
import com.ibm.commerce.order.facade.datatypes.OrderItemType;
import com.ibm.commerce.order.facade.datatypes.OrderType;
import com.ibm.commerce.order.facade.datatypes.ShippingModeIdentifierType;
import com.ibm.commerce.order.facade.datatypes.ShippingModeType;
import com.ibm.commerce.order.facade.server.commands.ComposeUsableShippingInformationByAddressCmdImpl;
import com.ibm.commerce.order.objects.OrderAccessBean;
import com.ibm.commerce.order.utils.AdminConfigLoader;
import com.ibm.commerce.order.utils.OrderThreshold;
import com.ibm.commerce.price.beans.FormattedMonetaryAmountDataBean;
import com.ibm.commerce.price.utils.MonetaryAmount;
import com.ibm.commerce.ras.ECMessage;
import com.ibm.commerce.user.beans.AddressDataBean;

/**
 * This compose task command is called from <code>GetOrderCmdImpl</code>.
 * It populates the order BOD using <code>OrderDataBean</code> and <code>OrderItemShippingModeListDataBean</code>. Only the 
 * shipping mode that is meaningful according to the shipping address will be returned for each order item. And the shipping
 * charge of each shipping mode will returned.
 * 
 * For example, "US-Overnight Delivery" will not be returned if the shipping address of the order item is in Australia, 
 * since according to this shipping mode and address, the shipping charge can not be calculated. But "International Regular" will be returned.
 * 
 * The following information about the noun is included: <br>
 * <pre>
 * <ul>
 *<li>/Order/OrderItem/UsableShippingMode/ShippingModeIdentifier/UniqueID</li>
 *<li>/Order/OrderItem/UsableShippingMode/Description</li>
 *<li>/Order/OrderItem/UsableShippingMode/UserData@amount</li>
 *<li>/Order/OrderItem/UsableShippingMode/UserData@currency</li>
 * </ul>
 * </pre>
 *  
 */

public class ComposeUsableShippingInfoWithShippingChargeCmdImpl extends
ComposeUsableShippingInformationByAddressCmdImpl  {

	 /**
     * IBM copyright notice field.
     */
    public static final String COPYRIGHT = com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;

    private static final String CLASSNAME = ComposeUsableShippingInfoWithShippingChargeCmdImpl.class.getName();

    private static final java.util.logging.Logger LOGGER = com.ibm.commerce.foundation.common.util.logging.LoggingHelper
            .getLogger(ComposeUsableShippingInfoWithShippingChargeCmdImpl.class);
    
    private static final AdminConfigLoader<OrderThreshold.Part> LOADER =
            new AdminConfigLoader<OrderThreshold.Part>(OrderFacadeConstants.COMPONENT_NAME);

    private static final String AMOUNT_USER_DATA_KEY  ="amount";
    
    private static final String CURRENCY_USER_DATA_KEY = "currency";
    
    private static final String ZERO_SHIP_CHARGE  = "0";
    /**
	 * 
	 * The special shipping mode code for pick up in store.
	 */
	
	private static final String IN_STORE_PICKUP_SHIPPING_MODE_CODE = "PickupInStore";
    
    /**
     * Composes the shipping information for each order item in {@link OrderType Order}<code>SDO</code> from the 
     * <code>OrderItemShippingModeListDataBean</code>.
     * 
     * The shipping mode will be filtered according to the shipping address of each order item. Only the 
	 * shipping mode that is meaningful according to the shipping address will be returned for each order item.
	 * 
	 * For example, "US-Overnight Delivery" will not be returned if the shipping address of the order item is in Australia, 
	 * since according to this shipping mode and address, the shipping charge can not be calculated. But "International Regular" will be returned.
	 * 
     * @param aabOrder
     *            the order access bean.
     * @return the {@link OrderType Order}<code>SDO</code>.
     * @throws ECException
     */
    protected OrderType composeUsableShippingInformationByAddress(OrderAccessBean aabOrder) throws ECException {
        final String METHODNAME = "composeUsableShippingInformationByAddress";
        if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
            LOGGER.entering(CLASSNAME, METHODNAME);
        }
        
        try {
            //Get Order level usable shipping address
            OrderDataBean dbOrder = new OrderDataBean();
            dbOrder.setCommandContext(getCommandContext());
            dbOrder.setOrderId(aabOrder.getOrderId().toString());
            dbOrder.populate();
            AddressDataBean[] orderAddBean = dbOrder.getAllowableShippingAddress();

            OrderType order = createOrderType();

            order.setOrderIdentifier(composeOrderIdentifier(aabOrder));

            /**
             * add if statements for defect 175102.
             */
            if (orderAddBean != null) {
                for (int i = 0; i < orderAddBean.length; i++) {
                    ContactInfoIdentifierType usableShippAdd = composeContactInfoIdentifier(orderAddBean[i]);
                    order.getUsableShippingAddress().add(usableShippAdd);
                }
            }

            //Get Order item level usable shipping address and mode
            OrderItemDataBean[] orderItemBeans = dbOrder.getOrderItemDataBeans();
            int start = 0;
            int end = 0;            
            if(orderItemBeans!=null){
            	end = orderItemBeans.length;
            }
            
            GetType get = null;
            if(getGetOrder()!=null && getGetOrder().getDataArea()!=null){
            	get = getGetOrder().getDataArea().getGet();
            }            
            
            if(get!=null){
            	String lastElement = getLastElementFromXPath(get);
            	boolean needPagingOnItem = OrderFacadeConstants.ELEMENT_ORDER_ITEM.equals(lastElement);		       	
            	
            	if (needPagingOnItem ) {
            		
            		BigInteger recordSetStartNumberInteger = get.getRecordSetStartNumber();
                    if (recordSetStartNumberInteger == null) {
                        recordSetStartNumberInteger = BigInteger.ZERO;
                    }

                    start = Math.max(0, Math.min(end, recordSetStartNumberInteger.intValue()));

                    BigInteger maxItems = get.getMaxItems();
                    int upperLimit = end - start;
                    BigInteger pageSize =
                        LOADER.getPageSizeThreshold(OrderThreshold.Part.SHOPPING_CART, maxItems, upperLimit);

                    end = Math.max(0, Math.min(end, start + pageSize.intValue()));        
                    
                    orderItemBeans = dbOrder.getOrderItemDataBeans(start,end);
                    
                    getPagingParameters().setRecordSetStartNumber(BigInteger.valueOf(start));
                    if(orderItemBeans!=null){
                    	getPagingParameters().setRecordSetCount(BigInteger.valueOf(orderItemBeans.length));
                    }else{
                    	getPagingParameters().setRecordSetCount(BigInteger.valueOf(0));
                    }           
                    getPagingParameters().setRecordSetTotal(BigInteger.valueOf(dbOrder.getTotalSize()));
                    getPagingParameters().setRecordSetCompleteIndicator(end >= dbOrder.getTotalSize()); 
                    
                }            	
            }
            if(orderItemBeans!=null){
            	ArrayList commonModeIdList = new ArrayList();
            	for (int i = 0; i < orderItemBeans.length; i++) {
                    OrderItemType orderItem = createOrderItemType();
                    order.getOrderItem().add(orderItem);
                    OrderItemIdentifierType orderItemIdent = getCommerceFoundationFactory()
                            .createOrderItemIdentifierType();
                    orderItem.setOrderItemIdentifier(orderItemIdent);
                    orderItem.getOrderItemIdentifier().setUniqueID(orderItemBeans[i].getOrderItemId());
                    
                    if(i == 0){
                    	OrderItemShippingInfoType oiShippingInfo = this.createOrderItemShippingInfoType();
                        ShippingModeType shipModeType = this.createShippingModeType();
                        ShippingModeIdentifierType shippingModeIdent = getOrderFactory().createShippingModeIdentifierType();
            	        shippingModeIdent.setUniqueID(orderItemBeans[i].getShippingModeId());
            	        shipModeType.setShippingModeIdentifier(shippingModeIdent);
                        oiShippingInfo.setShippingMode(shipModeType);
                        orderItem.setOrderItemShippingInfo(oiShippingInfo);
                    }
                    
                    AddressDataBean[] addresses = orderItemBeans[i].getAllowableShippingAddressDataBeans();
                    try{
	                    for (int j = 0; j < addresses.length; j++) {
	                        ContactInfoIdentifierType usableShippAdd = composeContactInfoIdentifier(addresses[j]);
	                        orderItem.getUsableShippingAddress().add(usableShippAdd);
	                    }
                    }catch (NullPointerException e) {
                        LOGGER.logp(LoggingHelper.DEFAULT_TRACE_LOG_LEVEL, CLASSNAME, METHODNAME, e.toString());
                    }
                    
                    try{
                    	//handle PickUpInStore
                    	ShippingModeDataBean[] usableShipModes = orderItemBeans[i].getAllowableShippingModeDataBeans();
                    	String pickUpInStoreShipModeId = null;
                    	for (int j = 0; j < usableShipModes.length ; j++){
                    		ShippingModeDataBean smDB = usableShipModes[j];
                    		if (smDB != null && smDB.getCode() != null && smDB.getCode().equals(IN_STORE_PICKUP_SHIPPING_MODE_CODE) && smDB.getCarrier() == null){
                    			pickUpInStoreShipModeId = smDB.getShippingModeId();
                    			break;
                    		}	
                    	}
                    	
                    	//OrderItemShippingModeListDataBean can filter the shipping mode if the "calculateShippingCharge" is set to true.
                    	OrderItemShippingModeListDataBean oiShipModeListBean = new OrderItemShippingModeListDataBean();
                        oiShipModeListBean.setOrderItemId(new Long(orderItemBeans[i].getOrderItemId()));
                        oiShipModeListBean.setCalculateShippingCharge(true);
            			com.ibm.commerce.beans.DataBeanManager.activate(oiShipModeListBean, getCommandContext());
            			ShippingModeDataBean[] shipModes = oiShipModeListBean.getShippingModes();
            			FormattedMonetaryAmountDataBean[]  shippingCharges = oiShipModeListBean.getShippingCharges();
            			ArrayList shipModeIdsForItem = new ArrayList();
	                    
            			if (pickUpInStoreShipModeId != null ){
	                    	MonetaryAmount monetaryAmount = new MonetaryAmount(new BigDecimal(ZERO_SHIP_CHARGE), orderItemBeans[i].getCurrency());
	                    	FormattedMonetaryAmountDataBean shippingCharge =new FormattedMonetaryAmountDataBean(
	                    			monetaryAmount,
	                    			getCommandContext().getStore(),
	                    			getCommandContext().getLanguageId());
	                        orderItem.getUsableShippingMode().add(composeShippingMode(pickUpInStoreShipModeId, shippingCharge));
	                        shipModeIdsForItem.add(pickUpInStoreShipModeId);
	                        if(i == 0){//first order item
	                        	commonModeIdList.add(pickUpInStoreShipModeId);
	                        } 
	                    }
	                    
            			
            			for (int k = 0; k < shipModes.length; k++) {
	                    	FormattedMonetaryAmountDataBean shippingCharge = shippingCharges[k];
	                        orderItem.getUsableShippingMode().add(composeShippingMode(shipModes[k].getShippingModeId(), shippingCharge));
	                        shipModeIdsForItem.add(shipModes[k].getShippingModeId());
	                        if(i == 0){//first order item
	                        	commonModeIdList.add(shipModes[k].getShippingModeId());
	                        } 
	                    }
	                    
	                    
                    	if(i > 0){//If not first order item, remove uncommon mode
                    		ArrayList toBeRemoved = new ArrayList();
    	                    for(int k=0; k<commonModeIdList.size();k++){
    	                    	String shipingModeId = (String)commonModeIdList.get(k);
    	                    	if(!shipModeIdsForItem.contains(shipingModeId)){
    	                    		//If shipping mode is not in the usable shipping mode of this item, to be removed.
    	                    		toBeRemoved.add(shipingModeId);
    	                    	}
    	                    }
    	                    commonModeIdList.removeAll(toBeRemoved);
                    	}
	                    
                    	
                    }catch (NullPointerException e) {
                        LOGGER.logp(LoggingHelper.DEFAULT_TRACE_LOG_LEVEL, CLASSNAME, METHODNAME, e.toString());
                    }
                    //Get usable shipping charge type list
                    Map policyAccessBeans = getBusinessPolicyAccessBeans(orderItemBeans[i].getContractId());
                    Iterator iter = policyAccessBeans.values().iterator();
                    while(iter.hasNext()){
                    	BusinessPolicyAccessBean policyAccessBean = (BusinessPolicyAccessBean)iter.next();
                    	orderItem.getUsableShippingChargePolicy().add(composeShippingChargePolicyInfo(policyAccessBean));
                    }
                }
            	if(commonModeIdList.size()>0){
            		for(int i=0;i<commonModeIdList.size();i++){
            			order.getUsableShippingMode().add(composeShippingMode((String)commonModeIdList.get(i)));
            		}
            		
            	}
            }

            if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
                LOGGER.exiting( CLASSNAME, METHODNAME, order);
            }

            return order;

        } catch (CreateException e) {
            throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, getClass().getName(), METHODNAME,
                    new Object[] { e.toString() }, e);
        } catch (FinderException e) {
            throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, getClass().getName(), METHODNAME,
                    new Object[] { e.toString() }, e);
        } catch (NamingException e) {
            throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, getClass().getName(), METHODNAME,
                    new Object[] { e.toString() }, e);
        } catch (RemoteException e) {
            throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, getClass().getName(), METHODNAME,
                    new Object[] { e.toString() }, e);
        } catch (Exception e) {
            if (LoggingHelper.isTraceEnabled(LOGGER)) {
                LOGGER.logp(LoggingHelper.DEFAULT_TRACE_LOG_LEVEL, CLASSNAME, METHODNAME, e.toString());
            }
            throw new ECApplicationException(ECMessage._ERR_GENERIC, getClass().getName(), METHODNAME,
                    new Object[] { e.getMessage() });
        }
    }
    private CommandContext getCommandContext() {
        CommandContext commandContext = null;
        ActivityToken newToken = ((BusinessContextInternalService) BusinessContextServiceFactory
                .getBusinessContextService()).getActivityToken();
        commandContext = ContextHelper.createCommandContext(newToken);
        return commandContext;
    }
    
    
    /**
     * Composes an {@link ShippingModeType} <code>SDO</code> from the shipping mode.
     * @param strShippingMode the shipping mode.
     * * @param shippingCharge the shipping charge.
     * @return the {@link ShippingModeType} <code>SDO</code>. 
     * @throws ECException
     */
    protected ShippingModeType composeShippingMode(String strShippingMode, FormattedMonetaryAmountDataBean shippingCharge) throws ECException{
        String methodName = "composeShippingMode";
        if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
            LOGGER.entering(CLASSNAME, methodName);
        }
        try{
        
	        ShippingModeType shippingMode = createShippingModeType();
	        ShippingModeIdentifierType shippingModeIdent = getOrderFactory().createShippingModeIdentifierType();
	        shippingModeIdent.setUniqueID(strShippingMode);
	        shippingMode.setShippingModeIdentifier(shippingModeIdent);
            
	        try{
	            DescriptionType desc = getCommerceFoundationFactory().createDescriptionType();
		        ShippingModeDescriptionAccessBean shipModeDesc 
		        	= ShippingModeDescriptionCache.findByPrimaryKey(
	        			String.valueOf(getCommandContext().getLanguageId()),
	        			strShippingMode);
	            desc.setValue(shipModeDesc.getDescription());
	            desc.setLanguage(shipModeDesc.getLanguageId());
	            shippingMode.setDescription(desc);
	            if (shippingCharge != null){
	            	UserDataType shipChargeUserData = shippingMode.getUserData();
	            	if (shipChargeUserData == null){
	            		shipChargeUserData = getCommerceFoundationFactory().createUserDataType();
	            	}
	            	Map<String, String> shippingChargeMap = new HashMap<String, String>();
	            	BigDecimal amount = shippingCharge.getAmount();
	            	String currency = shippingCharge.getCurrency();
	            	shippingChargeMap.put(AMOUNT_USER_DATA_KEY, String.valueOf(amount));
	            	shippingChargeMap.put(CURRENCY_USER_DATA_KEY, currency);
	            	shipChargeUserData.getUserDataField().putAll(shippingChargeMap);
	            	shippingMode.setUserData(shipChargeUserData);
	            }
	        } catch (FinderException ex) {
	            LOGGER.logp(LoggingHelper.DEFAULT_TRACE_LOG_LEVEL, CLASSNAME, methodName, ex.toString());
	        }
	        try {

	            ShippingModeAccessBean shippingModeBean 
	            	= ShippingModeCache.findByPrimaryKey(strShippingMode);

	            if (shippingMode.getShippingModeIdentifier() == null) {
	            	shippingMode.setShippingModeIdentifier(getOrderFactory().createShippingModeIdentifierType());
	            }
	            
	            if (shippingMode.getShippingModeIdentifier().getExternalIdentifier() == null) {
	            	shippingMode.getShippingModeIdentifier().setExternalIdentifier(getOrderFactory().createExternalShipModeIdentifier());
	            }
	            
	            if (shippingModeBean.getCode() != null) {
	            	shippingMode.getShippingModeIdentifier().getExternalIdentifier().setShipModeCode(shippingModeBean.getCode());
	            }
	            
	            if (shippingModeBean.getCarrier() != null) {
	            	shippingMode.getShippingModeIdentifier().getExternalIdentifier().setCarrier(shippingModeBean.getCarrier());
	            }
	        } catch (FinderException ex) {
	            LOGGER.logp(LoggingHelper.DEFAULT_TRACE_LOG_LEVEL, CLASSNAME, methodName, ex.toString());
	        }

            if (LoggingHelper.isEntryExitTraceEnabled(LOGGER)) {
                LOGGER.exiting( CLASSNAME, methodName, shippingMode);
            }

	        return shippingMode;
        } catch (CreateException ex) {
            throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASSNAME, methodName, ex);
        } catch (NamingException ex) {
            throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, CLASSNAME, methodName, ex);
        } catch (RemoteException ex) {
            throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASSNAME, methodName, ex);
        }

    }
    
   
}
