package com.ibm.commerce.applepay.payment.commands;

/*
 *-----------------------------------------------------------------
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * WebSphere Commerce
 *
 * (C) Copyright IBM Corp. 2016
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 *-----------------------------------------------------------------
 */

import java.rmi.RemoteException;
import java.util.ArrayList;

import javax.ejb.CreateException;
import javax.ejb.FinderException;
import javax.naming.NamingException;

import com.ibm.commerce.command.ControllerCommandImpl;
import com.ibm.commerce.datatype.TypedProperty;
import com.ibm.commerce.edp.api.InternalException;
import com.ibm.commerce.edp.api.InvalidDataException;
import com.ibm.commerce.edp.api.J2EEException;
import com.ibm.commerce.edp.utils.ExceptionHelper;
import com.ibm.commerce.exception.ECException;
import com.ibm.commerce.exception.ECSystemException;
import com.ibm.commerce.payment.rules.EDPServices;
import com.ibm.commerce.payments.plugincontroller.beans.MerchantConfAccessBean;
import com.ibm.commerce.payments.plugincontroller.beans.MerchantConfInfoAccessBean;
import com.ibm.commerce.payments.plugincontroller.beans.PluginController;
import com.ibm.commerce.payments.plugincontroller.beans.StoreMerchantAccessBean;
import com.ibm.commerce.ras.ECMessage;
import com.ibm.commerce.ras.ECTrace;
import com.ibm.commerce.ras.ECTraceIdentifiers;

/**
 * Get merchant information.
 */
public class GetMerchantInfoCmdImpl extends ControllerCommandImpl implements
		GetMerchantInfoCmd {
	
	public static final String CLASS_NAME = GetMerchantInfoCmdImpl.class.getName();
	
	private String targetStoreId;
	private String paymentSystem;
	private String paymentConfigGroup;
	
	public static final String STORE_ID = "storeId";
	public static final String PAYMENT_SYSTEM = "paymentSystem";
	public static final String PAYMENT_CONFIG_GROUP = "paymentConfigGroup";
	public static final String PAYMENT_SYSTEM_APPLEPAY = "ApplePay";
	public static final String PAYMENT_DEFAULT_CONFIG_GROUP = "default";
	
	@Override
	public void setRequestProperties(com.ibm.commerce.datatype.TypedProperty requestProperties) throws com.ibm.commerce.exception.ECException {
		final String METHOD_NAME = "setRequestProperties";
		Object[] param = {requestProperties};
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_PPC, CLASS_NAME, METHOD_NAME, param);
		
		super.setRequestProperties(requestProperties);
		
		this.targetStoreId = requestProperties.getString(STORE_ID, null);
		this.paymentSystem = requestProperties.getString(PAYMENT_SYSTEM, null);
		this.paymentConfigGroup = requestProperties.getString(PAYMENT_CONFIG_GROUP, null);
		
		if(paymentSystem == null || paymentSystem ==""){
			paymentSystem = PAYMENT_SYSTEM_APPLEPAY;
		}
		
		if(paymentConfigGroup == null || paymentConfigGroup ==""){
			paymentConfigGroup = PAYMENT_DEFAULT_CONFIG_GROUP;
		}
		
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_PPC, CLASS_NAME, METHOD_NAME);
	}

	@Override
	public void performExecute() throws ECException{
		final String METHOD_NAME = "performExecute()";
		ECTrace.entry(ECTraceIdentifiers.COMPONENT_PPC, CLASS_NAME, METHOD_NAME);
		
		try {
			responseProperties = new TypedProperty();
			
			PluginController paymentFacade = getPaymentsFacade();
			StoreMerchantAccessBean storeMerchAB = paymentFacade.getStoreMerchantByStoreId(Integer.valueOf(this.getTargetStoreId()));
			Long merchantId = storeMerchAB.getMerchantId();
			ArrayList results = (ArrayList) paymentFacade.getMerchantConf(merchantId, this.getPaymentSystem(), this.getPaymentConfigGroup());
			if(results.size() > 0){
				MerchantConfAccessBean merchConfAB = (MerchantConfAccessBean) results.get(0);
				Long merchantConfigId = merchConfAB.getMerchantConfId();
				ArrayList configInfos = (ArrayList) paymentFacade.getMerchantConfInfo(merchantConfigId);
				if(configInfos.size() >0){
					for(int i=0; i<configInfos.size(); i++){
						MerchantConfInfoAccessBean merchConfInfoAB = (MerchantConfInfoAccessBean) configInfos.get(i);
						String key = merchConfInfoAB.getPropertyName().trim();
						String value = merchConfInfoAB.getPropertyValue().trim();
						
						if(value.contains(",")){
							String[] valueArray = value.split(",");
							for(int j=0;j<valueArray.length;j++){
								valueArray[j] = valueArray[j].trim(); 
							}
							responseProperties.put(key, valueArray);
							continue;
						}

						responseProperties.put(key, value);
						ECTrace.trace(ECTraceIdentifiers.COMPONENT_PPC, CLASS_NAME, METHOD_NAME, "MERCHCONFINFO: PropertyName:'"+key+"', value:'"+value+"'.");
					}
				}
		
			}else{
				ECTrace.trace(ECTraceIdentifiers.COMPONENT_PPC, CLASS_NAME, METHOD_NAME, "No result is found from merchant config for merchant with ID:"+merchantId);
			}
		} catch (J2EEException e) {
			throw ExceptionHelper.buildECException(e, CLASS_NAME, METHOD_NAME);
		} catch (InvalidDataException e) {
			throw ExceptionHelper.buildECException(e, CLASS_NAME, METHOD_NAME);
		} catch (InternalException e) {
			throw ExceptionHelper.buildECException(e, CLASS_NAME, METHOD_NAME);
		} catch (NumberFormatException e) {
			throw new ECSystemException(ECMessage._ERR_NUMBER_FORMAT_EXCEPTION, CLASS_NAME, METHOD_NAME, e);
		} catch (RemoteException e) {
			throw new ECSystemException(ECMessage._ERR_REMOTE_EXCEPTION, CLASS_NAME, METHOD_NAME, e);
		} catch (CreateException e) {
			throw new ECSystemException(ECMessage._ERR_CREATE_EXCEPTION, CLASS_NAME, METHOD_NAME, e);
		} catch (FinderException e) {
			throw new ECSystemException(ECMessage._ERR_FINDER_EXCEPTION, CLASS_NAME, METHOD_NAME, e);
		} catch (NamingException e) {
			throw new ECSystemException(ECMessage._ERR_NAMING_EXCEPTION, CLASS_NAME, METHOD_NAME, e);
		}
		
		
		ECTrace.exit(ECTraceIdentifiers.COMPONENT_PPC, CLASS_NAME, METHOD_NAME);
	}
	
	/**
	 * This method gets interface of the payment processing component. You
	 * should first call this method to get the interface of the payment
	 * processing component. Then call the API of the interface to execute
	 * the corresponding payment/refund actions.
	 * 
	 * @return PluginController The interface of the payment processing component.
	 * @throws J2EEException
	 */
	protected PluginController getPaymentsFacade() throws J2EEException {
		return EDPServices.getServices().getPaymentsAccessor().getPayments();
	}

	/**
	 * @return the targetStoreId
	 */
	public String getTargetStoreId() {
		return targetStoreId;
	}

	/**
	 * @return the paymentSystem
	 */
	public String getPaymentSystem() {
		return paymentSystem;
	}

	/**
	 * @return the paymentConfigGroup
	 */
	public String getPaymentConfigGroup() {
		return paymentConfigGroup;
	}	

}
