package com.ibm.commerce.applepay.payment.commands;

/*
 *-----------------------------------------------------------------
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * WebSphere Commerce
 *
 * (C) Copyright IBM Corp. 2016
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 *-----------------------------------------------------------------
 */

import java.util.Map;

import com.ibm.commerce.applepay.payment.objects.PKPaymentToken;
import com.ibm.commerce.command.TaskCommand;
import com.ibm.commerce.datatype.TypedProperty;
import com.ibm.commerce.order.facade.datatypes.OrderType;
import com.ibm.commerce.payments.plugin.FinancialTransaction;

/**
 *  This command is invoked by ApplePayBean to process the payment token from ApplePay.
 */
public interface ProcessApplePayPaymentCmd extends TaskCommand {
	

	/**
	 * The IBM copyright notice field.
	 */
	public static final String COPYRIGHT				= com.ibm.commerce.copyright.IBMCopyright.SHORT_COPYRIGHT;
	/**
	 * The name of this command.
	 */
	public static final String NAME						= ProcessApplePayPaymentCmd.class.getCanonicalName();
	/**
	 * The default implementation of this command.
	 */
	public static final String defaultCommandClassName	= NAME + "Impl";
	
	/**
	 * Set the payment token returned from ApplePay.
	 * @param paymentToken
	 */
	public void setPaymentToken(PKPaymentToken paymentToken);
	
	/**
	 * Get the payment token
	 * @return paymentToken
	 */
	public PKPaymentToken getPaymentToken();
	
	/**
	 * Set the payment action, eg. Approve, Deposit etc.
	 * @param action
	 */
	public void setPaymentAction(String action);
	
	/**
	 * Get the payment action, eg. Approve, Deposit etc. 
	 * @return action
	 */
	public String getPaymentAction();
	
	/**
	 * Set merchant information.
	 * @param merchantInfo
	 */
	public void setMerchantInfo(TypedProperty merchantInfo);
	
	/**
	 * Get merchant information.
	 * @return financialTransaction
	 */
	public Map getMerchantInfo();
	
	/**
	 * Set the FinancialTransaction for the given payment action.
	 * @param financialTransaction
	 */
	public void setFinancialTransaction(FinancialTransaction financialTransaction);
	
	/**
	 * Get the FinancialTransaction result.
	 * @return financialTransaction
	 */
	public FinancialTransaction getFinancialTransaction();
	
	/**
	 * Set the order noun object.
	 * @param orderNoun
	 */
	public void setOrderNoun(OrderType orderNoun);
	
	/**
	 * Get the order noun object.
	 * @return orderNoun
	 */
	public OrderType getOrderNoun();
	
	/**
	 * Set the extended parameters.
	 * @param extParams
	 */
	public void setExtendedParameters(TypedProperty extParams);
	
	/**
	 * Get the extended parameters.
	 * @return extParams
	 */
	public TypedProperty getExtendedParameters();
	
}
