package com.ibm.commerce.applepay.payment.ipg;

/*
 *-----------------------------------------------------------------
 * IBM Confidential
 *
 * OCO Source Materials
 *
 * WebSphere Commerce
 *
 * (C) Copyright IBM Corp. 2016
 *
 * The source code for this program is not published or otherwise
 * divested of its trade secrets, irrespective of what has
 * been deposited with the U.S. Copyright Office.
 *-----------------------------------------------------------------
 */

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import com.ibm.commerce.applepay.payment.commands.ProcessApplePayPaymentCmd;
import com.ibm.commerce.applepay.payment.commands.ProcessApplePayPaymentCmdImpl;
import com.ibm.commerce.applepay.payment.objects.PKPaymentToken;
import com.ibm.commerce.order.facade.datatypes.OrderType;
import com.ibm.commerce.payments.plugin.FinancialTransaction;
import com.ibm.commerce.payments.plugin.PluginConstants;
import com.ibm.commerce.ras.ECTrace;
import com.ibm.commerce.ras.ECTraceIdentifiers;

/**
 * This implementation connects IPG for ApplePay payment processing.
 */
public class ProcessApplePayPaymentIPGCmdImpl extends
		ProcessApplePayPaymentCmdImpl implements ProcessApplePayPaymentCmd {
	
	
	public static final String CLASS_NAME = ProcessApplePayPaymentIPGCmdImpl.class.getName();
	private static final Long TRACE_COMPONENT = ECTraceIdentifiers.COMPONENT_PPC;
	
	/**
	 * This method is used for executing the Approve action.
	 */
	protected void approve(){
		String result = null;
		final String METHOD_NAME = "approve()";
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		
		
		/**
		 * The common steps for the business logic:
		 * 1. Compose the necessary request properties including the payment token
		 * 2. Send the payment request to actual Payment Gateway using SDK or API with action code
		 * 3. Analyze payment result from Payment Gateway	
		 * 4. Send back to ApplePayPlugin by setting the result from Gateway to:
		 * 		com.ibm.commerce.applepay.payment.commands.ProcessApplePayPaymentCmdImpl.financialTransaction
		 */
		
		//send payment request to IPG
		try {
			PKPaymentToken token = this.getPaymentToken();
			OrderType order = this.getOrderNoun();
			FinancialTransaction financialTransaction = this.getFinancialTransaction();
			
			String response = IPGUtils.sendRequest(token, order);
			Document doc = IPGUtils.convertXMLStringToDOM(response);
			
			String xpath_returnCode = "/TRX/RES/RCD";
			XPathFactory factory = XPathFactory.newInstance();
			XPath xpath = factory.newXPath();
			XPathExpression expr = xpath.compile(xpath_returnCode);
			Node returnCode_node = (Node) expr.evaluate(doc, XPathConstants.NODE);
			String returnCode = "";
			if(returnCode_node!=null){
				returnCode = returnCode_node.getTextContent();
			}
			
			String xpath_authCode = "/TRX/AUT/AUC";
			expr = xpath.compile(xpath_authCode);
			Node authCode_node = (Node) expr.evaluate(doc, XPathConstants.NODE);
			String auc = null;
			if(authCode_node!=null){
				auc = authCode_node.getTextContent();
			}
			if(returnCode.trim().equalsIgnoreCase("0")){
				financialTransaction.setState(FinancialTransaction.STATE_SUCCESS);
				financialTransaction.setReasonCode(PluginConstants.RESPONSECODE_SUCCESS);
				financialTransaction.setProcessedAmount(order.getOrderAmount().getGrandTotal().getValue());
				if(auc!=null){
					financialTransaction.setReferenceNumber(auc);					
				}
			}else{
				financialTransaction.setState(FinancialTransaction.STATE_FAILED);
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
		
		ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
	}
	
	/**
	 * This method is used for executing the Deposit action.
	 */
	protected void deposit(){
		String result = null;
		final String METHOD_NAME = "deposit()";
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		
		/**
		 * The common steps for the business logic:
		 * 1. Compose the necessary request properties including the payment token
		 * 2. Send the payment request to actual Payment Gateway using SDK or API with action code
		 * 3. Analyze payment result from Payment Gateway	
		 * 4. Send back to ApplePayPlugin by setting the result from Gateway to:
		 * 		com.ibm.commerce.applepay.payment.commands.ProcessApplePayPaymentCmdImpl.financialTransaction
		 */

		
		ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
	}

	/**
	 * This method is used for executing the Deposit action.
	 */
	protected void approveAndDeposit(){
		String result = null;
		final String METHOD_NAME = "approveAndDeposit()";
		ECTrace.entry(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
		
		/**
		 * The common steps for the business logic:
		 * 1. Compose the necessary request properties including the payment token
		 * 2. Send the payment request to actual Payment Gateway using SDK or API with action code
		 * 3. Analyze payment result from Payment Gateway	
		 * 4. Send back to ApplePayPlugin by setting the result from Gateway to:
		 * 		com.ibm.commerce.applepay.payment.commands.ProcessApplePayPaymentCmdImpl.financialTransaction
		 */

		
		ECTrace.exit(TRACE_COMPONENT, CLASS_NAME, METHOD_NAME);
	}


}
